@@ -10,7 +10,6 @@ Lingua::EN::NameParse - routines for manipulating a person's name
     my %args =
     (
         auto_clean      => 1,
-        force_case      => 1,
         lc_prefix       => 1,
         initials        => 3,
         allow_reversed  => 1,
@@ -18,19 +17,12 @@ Lingua::EN::NameParse - routines for manipulating a person's name
         extended_titles => 0
     );
 
-    my $name = new Lingua::EN::NameParse(%args);
+    my $name = Lingua::EN::NameParse->new(%args);
 
     $error = $name->parse("MR AC DE SILVA");
-    if ( $error )
-    {
-        $bad_input = $my_properties{non_matching};
-    }
-    else
+    unless ( $error )    
     {
         %name_comps = $name->components;
-        $surname = $name_comps{surname_1}; # DE SILVA
-
-        %name_comps = $name->case_components;
         $surname = $name_comps{surname_1}; # De Silva
 
         $correct_casing = $name->case_all; # Mr AC de Silva
@@ -101,19 +93,18 @@ in peoples names, such as Mr AB McNay.
 To describe the formats supported by NameParse, a short hand representation
 of the name is used. The following formats are currently supported :
 
-    Mr_John_Smith_&Ms_Mary_Jones
-    Mr_A_Smith_&Ms_B_Jones
-    Mr_&Ms_A_&B_Smith
-    Mr_A_&Ms_B_Smith
-    Mr_&Ms_A_Smith
-    Mr_A_&B_Smith
-    John_Smith_&Mary_Jones
-    John_&Mary_Smith
-    A_Smith_&B_Jones
+    Mr_John_Smith_&_Ms_Mary_Jones
+    Mr_A_Smith_&_Ms_B_Jones
+    Mr_&Ms_A_&_B_Smith
+    Mr_A_&_Ms_B_Smith
+    Mr_&_Ms_A_Smith
+    Mr_A_&_B_Smith
+    John_Smith_&_Mary_Jones
+    John_&_Mary_Smith
+    A_Smith_&_B_Jones
 
     Mr_John_Adam_Smith
     Mr_John_A_Smith
-    Mr_J_Adam_Smith
     Mr_John_Smith
     Mr_A_Smith
     John_Adam_Smith
@@ -145,28 +136,17 @@ usage.
    my %args =
    (
       auto_clean     => 1,
-      force_case     => 1,
       lc_prefix      => 1,
       initials       => 3,
       allow_reversed => 1
    );
 
 
-   my $name = new Lingua::EN::NameParse(%args);
+   my $name = Lingua::EN::NameParse->new(%args);
 
 
 =over 4
 
-=item force_case
-
-This option will force the C<case_all> method to name case the entire input
-string, including any unmatched sections that failed parsing. For example, in
-"MR A JONES & ASSOCIATES", "& ASSOCIATES" will also be name cased. The casing
-rules for unmatched sections are the same as for surnames. This is usually
-the best option, although any initials in the unmatched section will not
-be correctly cased. This option is useful when you know you data has invalid
-names, but you cannot filter out or reject them.
-
 =item auto_clean
 
 When this option is set to a positive value, any call to the C<parse> method
@@ -177,7 +157,7 @@ unprintable or non alphabetic characters.
 =item lc_prefix
 
 When this option is set to a positive value, it will force the C<case_all>
-and C<case_component> methods to lower case the first letter of each word that
+and C<components> methods to lower case the first letter of each word that
 occurs in the prefix portion of a surname. For example, Mr AB de Silva,
 or Ms AS von der Heiden.
 
@@ -259,7 +239,7 @@ capitals and the remainder to lower case, with the following exceptions-
    - see C<surname_prefs.txt> for user defined exceptions
 
 A complete definition of the capitalising rules can be found by studying
-the component grammar defined within the code.
+the case_surname function.
 
 The method returns the entire cased name as text.
 
@@ -276,13 +256,13 @@ This is useful for sorting names alphabetically by surname.
 The method returns the entire reverse order cased name as text.
 
 
-=head2 case_components
+=head2 components
 
-   %my_name = $name->case_components;
+   %my_name = $name->components;
    $cased_surname = $my_name{surname_1};
 
 
-The C<case_components> method does the same thing as the C<case_all> method,
+The C<components> method does the same thing as the C<case_all> method,
 but returns the name cased components in a hash. The following keys are used
 for each component:
 
@@ -300,23 +280,13 @@ for each component:
    surname_2
    suffix
 
-If a component has no matching data for a given name, it's values will be
-set to the empty string.
+If a component has no matching data for a given name, it will not appear in the hash
 
 If the name could not be parsed, this method returns null. If you assign the return
 value to a hash, you should check the error status returned by the C<parse> method first.
-Ohterwise, you will get an odd number of values addigned to the hash.
+Ohterwise, you will get an odd number of values assigned to the hash.
 
 
-=head2 components
-
-   %name = $name->components;
-   $surname = $my_name{surname_1};
-
-The C<components> method does the same thing as the C<case_components> method,
-but each component is returned as it appears in the input string, with no case
-conversion.
-
 =head2 case_surname
 
    $correct_casing = case_surname("DE SILVA-MACNAY" [,$lc_prefix]);
@@ -407,7 +377,6 @@ The type of format a name is in, as one of the following strings:
     Mr_A_&B_Smith
     Mr_John_Adam_Smith
     Mr_John_A_Smith
-    Mr_J_Adam_Smith
     Mr_John_Smith
     Mr_A_Smith
     John_Adam_Smith
@@ -483,12 +452,6 @@ Data Dictionary for transfer of street addressing information"
 
 =head1 FUTURE DIRECTIONS
 
-   Add filtering of very long names
-   Add diagnostic messages explaining why parsing failed
-   Add transforming methods to do things like remove dots from initials
-   Try to derive gender (Mr... is male, Ms, Mrs... is female)
-
-
 Define grammar for other languages. Hopefully, all that would be needed is
 to specify a new module with its own grammar, and inherit all the existing
 methods. I don't have the knowledge of the naming conventions for non-english
@@ -501,14 +464,8 @@ L<Lingua::EN::AddressParse>, L<Lingua::EN::MatchNames>, L<Lingua::EN::NickNames>
 L<Lingua::EN::NameCase>, L<Parse::RecDescent>
 
 
-=head1 TO DO
-
-
 =head1 BUGS
 
-The dot in a suffix of Jnr. or Snr. will be consumed as unmatched text,
-and not be retained with the suffix.
-
 Names with accented characters (acute, circumfelx etc) will not be parsed
 correctly. A work around is to replace the character class [a-z] with \w
 in the appropriate rules in the grammar tree, but this could lower the accuracy
@@ -518,7 +475,6 @@ of names based purely on ASCII text.
 
 Thanks to all the people who provided ideas and suggestions, including -
 
-   QM Industries <http://www.qmi.com.au>
    Damian Conway,  author of Parse::RecDescent
    Mark Summerfield author of Lingua::EN::NameCase,
    Ron Savage, Alastair Adam Huffman, Douglas Wilson
@@ -530,7 +486,7 @@ NameParse was written by Kim Ryan <kimryan at cpan dot org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2013 Kim Ryan. All rights reserved.
+Copyright (c) 2015 Kim Ryan. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
@@ -549,7 +505,7 @@ use Parse::RecDescent;
 use Exporter;
 use vars qw (@ISA @EXPORT_OK);
 
-our $VERSION = '1.32';
+our $VERSION = '1.33';
 @ISA       = qw(Exporter);
 @EXPORT_OK = qw(clean case_surname);
 
@@ -602,25 +558,26 @@ sub parse
         $input_string = join(' ',$second,$first);
     }
 
-    $name->{components} = ();
+    $name->{comps} = ();
     $name->{properties} = ();
     $name->{properties}{type} = 'unknown';
     $name->{error} = 0;
+    $name->{error_desc} = '';
+    $name->{warning} = 0;
+    $name->{warning_desc} = '';
 
+    $name->{original_input} = $input_string;
     $name->{input_string} = $input_string;
 
     $name = _pre_parse($name);
     unless ( $name->{error} )
     {
-        $name = _assemble($name);
-        _validate($name);
-
-        if ( $name->{error} and $name->{auto_clean} )
+        if ( $name->{auto_clean} )
         {
             $name->{input_string} = clean($name->{input_string});
-            $name = _assemble($name);
-            _validate($name);
-        }
+        }        
+        $name = _assemble($name);
+        _validate($name);
     }
 
    return($name->{error});
@@ -640,36 +597,17 @@ sub clean
 
     # remove any remaining leading or trailing space
     $input_string =~ s/^ //;
-    $input_string =~ s/ $//;
 
     return($input_string);
 }
+
 #-------------------------------------------------------------------------------
-# Given a name object, returns all components in a hash.
-# Else return no value. The error flag set in the parse method should be
-# tested first to ensure only key/value pairs are assigned to the hash
+# Given a name object, apply correct capitalisation to each component of a person's name.
+# Return all cased components in a hash.
+# Else return no value. 
+
 
 sub components
-{
-    my $name = shift;
-    if ( $name->{properties}{type} eq 'unknown'  )
-    {
-        return;
-    }
-    else
-    {
-        return(%{ $name->{components} });
-    }
-}
-#-------------------------------------------------------------------------------
-# Given a name object, apply correct capitalisation to each component of a
-# person's name.
-# Return all cased components in a hash.
-# Else return no value. The error flag set in the parse method should be
-# tested to ensure only key/value pairs are assigned to the hash
-
-
-sub case_components
 {
     my $name = shift;
 
@@ -679,7 +617,7 @@ sub case_components
     }
     else
     {
-        my %orig_components = $name->components;
+        my %orig_components = %{ $name->{comps} };
 
         my ($current_key,%cased_components);
         foreach $current_key ( keys %orig_components )
@@ -693,6 +631,10 @@ sub case_components
             {
                $cased_value = case_surname($orig_components{$current_key},$name->{lc_prefix});
             }
+            elsif ( $current_key eq 'type')
+            {
+               $cased_value = $orig_components{$current_key};
+            }            
             else
             {
                 $cased_value = _case_word($orig_components{$current_key});
@@ -713,19 +655,18 @@ sub case_components
 
 my %component_order=
 (
-    'Mr_John_Smith_&Ms_Mary_Jones' => ['title_1','given_name_1','surname_1','conjunction_1','title_2','given_name_2','surname_2'],
-    'Mr_A_Smith_&Ms_B_Jones' => ['title_1','initials_1','surname_1','conjunction_1','title_2','initials_2','surname_2'],
-    'Mr_&Ms_A_&B_Smith'     => ['title_1','conjunction_1','title_2','initials_1','conjunction_1','initials_2','surname_1'],
-    'Mr_A_&Ms_B_Smith'       => ['title_1','initials_1','conjunction_1','title_2','initials_2','surname_1'],
-    'Mr_&Ms_A_Smith'         => ['title_1','conjunction_1','title_2','initials_1','surname_1'],
-    'Mr_A_&B_Smith'          => ['title_1','initials_1','conjunction_1','initials_2','surname_1'],
+    'Mr_John_Smith_&_Ms_Mary_Jones' => ['title_1','given_name_1','surname_1','conjunction_1','title_2','given_name_2','surname_2'],
+    'Mr_A_Smith_&_Ms_B_Jones' => ['title_1','initials_1','surname_1','conjunction_1','title_2','initials_2','surname_2'],
+    'Mr_&_Ms_A_&_B_Smith'     => ['title_1','conjunction_1','title_2','initials_1','conjunction_2','initials_2','surname_1'],
+    'Mr_A_&_Ms_B_Smith'       => ['title_1','initials_1','conjunction_1','title_2','initials_2','surname_1'],
+    'Mr_&_Ms_A_Smith'         => ['title_1','conjunction_1','title_2','initials_1','surname_1'],
+    'Mr_A_&_B_Smith'          => ['title_1','initials_1','conjunction_1','initials_2','surname_1'],
     'John_Smith_&Mary_Jones' => ['given_name_1','surname_1','conjunction_1','given_name_2','surname_2'],
-    'John_&Mary_Smith'       => ['given_name_1','conjunction_1','given_name_2','surname_1'],
-    'A_Smith_&B_Jones'       => ['initials_1','surname_1','conjunction_1','initials_2','surname_2'],
+    'John_&_Mary_Smith'       => ['given_name_1','conjunction_1','given_name_2','surname_1'],
+    'A_Smith_&_B_Jones'       => ['initials_1','surname_1','conjunction_1','initials_2','surname_2'],
 
     'Mr_John_Adam_Smith'      => ['precursor','title_1','given_name_1','middle_name','surname_1','suffix'],
     'Mr_John_A_Smith'         => ['precursor','title_1','given_name_1','initials_1','surname_1','suffix'],
-    'Mr_J_Adam_Smith'         => ['precursor','title_1','initials_1','middle_name','surname_1','suffix'],
     'Mr_John_Smith'           => ['precursor','title_1','given_name_1','surname_1','suffix'],
     'Mr_A_Smith'              => ['precursor','title_1','initials_1','surname_1','suffix'],
     'John_Adam_Smith'         => ['precursor','given_name_1','middle_name','surname_1','suffix'],
@@ -740,15 +681,14 @@ my %component_order=
 # only include names with a single surname
 my %reverse_component_order=
 (
-   'Mr_&Ms_A_&B_Smith'  => ['surname_1','title_1','conjunction_1','title_2','initials_1','conjunction_1','initials_2'],
-   'Mr_A_&Ms_B_Smith'    => ['surname_1','title_1','initials_1','conjunction_1','title_2','initials_2'],
-   'Mr_&Ms_A_Smith'      => ['surname_1','title_1','title_1','conjunction_1','title_2','initials_1'],
-   'Mr_A_&B_Smith'       => ['surname_1','title_1','initials_1','conjunction_1','initials_2'],
-   'John_&Mary_Smith'    => ['surname_1','given_name_1','conjunction_1','given_name_2'],
+   'Mr_&_Ms_A_&_B_Smith'  => ['surname_1','title_1','conjunction_1','title_2','initials_1','conjunction_1','initials_2'],
+   'Mr_A_&_Ms_B_Smith'    => ['surname_1','title_1','initials_1','conjunction_1','title_2','initials_2'],
+   'Mr_&_Ms_A_Smith'      => ['surname_1','title_1','title_1','conjunction_1','title_2','initials_1'],
+   'Mr_A_&_B_Smith'       => ['surname_1','title_1','initials_1','conjunction_1','initials_2'],
+   'John_&_Mary_Smith'    => ['surname_1','given_name_1','conjunction_1','given_name_2'],
 
    'Mr_John_Adam_Smith'   => ['surname_1','title_1','given_name_1','middle_name','suffix'],
    'Mr_John_A_Smith'      => ['surname_1','title_1','given_name_1','initials_1','suffix'],
-   'Mr_J_Adam_Smith'      => ['surname_1','title_1','initials_1','middle_name','suffix'],
    'Mr_John_Smith'        => ['surname_1','title_1','given_name_1','suffix'],
    'Mr_A_Smith'           => ['surname_1','title_1','initials_1','suffix'],
    'John_Adam_Smith'      => ['surname_1','given_name_1','middle_name','suffix'],
@@ -781,7 +721,7 @@ sub case_all
         return;
     }
 
-    my %component_vals = $name->case_components;
+    my %component_vals = $name->components;
     my @order = @{ $component_order{$name->{properties}{type}} };
 
     foreach my $component_key ( @order )
@@ -794,12 +734,12 @@ sub case_all
            push(@cased_name,$component_vals{$component_key});
         }
     }
-    if ( $name->{error} and $name->{force_case} )
+    if ( $name->{comps}{non_matching} )
     {
        # Despite errors, try to name case non-matching section. As the format
        # of this section is unknown, surname case will provide the best
        # approximation, but still fail on initials of more than 1 letter
-       push(@cased_name,case_surname($name->{properties}{non_matching},$name->{lc_prefix}));
+       push(@cased_name,case_surname($name->{comps}{non_matching},$name->{lc_prefix}));
     }
 
     return(join(' ',@cased_name));
@@ -834,7 +774,7 @@ sub case_all_reversed
             # this type of name should not be reversed, such as two surnames
             return;
         }
-        my %component_vals = $name->case_components;
+        my %component_vals = $name->components;
         my @reverse_order = @{ $reverse_component_order{$name->{properties}{type} } };
 
         foreach my $component_key ( @reverse_order )
@@ -891,7 +831,7 @@ sub case_surname
 
     unless ($surname)
     {
-        return;
+        return('');
     }
 
     # If the user has specified a preferred capitalisation for this
@@ -999,13 +939,13 @@ sub salutation
     if
     (
         $name->{error} or
-        ( $name->{components}{precursor} and  $name->{components}{precursor} =~ /Estate/i)
+        ( $name->{comps}{precursor} and  $name->{comps}{precursor} =~ /ESTATE/)
     )
     {
         # Despite an error, the presence of a conjunction probably
         # means we are dealing with 2 or more people.
         # For example Mr AB Smith & John Jones
-        if ( $name->{input_string} =~ / (And|&) /i )
+        if ( $name->{input_string} =~ / (AND|&) / )
         {
            $sal_default .= 's';
         }
@@ -1013,7 +953,7 @@ sub salutation
     }
     else
     {
-        my %component_vals = $name->case_components;
+        my %component_vals = $name->components;
 
         if ( $sal_type eq 'given_name')
         {
@@ -1104,33 +1044,39 @@ sub properties
 sub report
 {
     my $name = shift;
-
-    printf("%-17.17s : %-40.40s\n","Input",$name->{input_string});
-
+    
     my %props = $name->properties;
+    
+    my $fmt = "%-20.20s : %s\n";
+
+    printf($fmt,"Original Input",$name->{original_input});
+    printf($fmt,"Cleaned Input",$name->{input_string});
+    printf($fmt,"Case all",$name->case_all);
+    printf($fmt,"Case all reversed",$name->case_all_reversed);
+    printf($fmt,"Salutation",$name->salutation(salutation => 'Dear',sal_default => 'Friend', sal_type => 'title_plus_surname'));
+    printf($fmt,"Type", $props{type});
+    printf($fmt,"Parsing Error", $name->{error});
+    printf($fmt,"Error description : ", $name->{error_desc});
+    printf($fmt,"Parsing Warning", $name->{warning});
+    printf($fmt,"Warning description", $name->{warning_desc});    
+    
+ 
     unless ($props{type} eq 'unknown')
     {
-        my %comps = $name->case_components;
+        my %comps = $name->components;
         if ( %comps )
         {
-            foreach my $comp ( sort keys %comps)
+            print("\nCOMPONENTS\n");
+            foreach my $value ( sort keys %comps)
             {
-                printf("%-17.17s : %s\n",$comp,$comps{$comp});
+                 if ($value and $comps{$value})
+                {
+                    printf($fmt,$value,$comps{$value});
+                }
             }
         }
     }
-
-    if ( $props{type} )
-    {
-        printf("%-17.17s : %-40.40s\n","Name type",$props{type});
-    }
-
-    if ( $props{non_matching} )
-    {
-        printf("%-17.17s : %-40.40s\n","Parsing Error","Yes");
-        printf("%-17.17s : %-40.40s\n","Non matching part",$props{non_matching});
-    }
-}
+}    
 #-------------------------------------------------------------------------------
 
 # PRIVATE METHODS
@@ -1140,18 +1086,27 @@ sub report
 sub _pre_parse
 {
     my $name = shift;
+    
+    # strip all full stops    
+    $name->{input_string}  =~ s/\.//g;
+    
+    # Fold all text to upper case, as these are used in all regular expressions withun thr grammar tree
+    $name->{input_string}  = uc($name->{input_string});
+    
     # Check that common reserved word (as found in company names) do not appear
     if ( $name->{input_string} =~
-         /\bPty\.? Ltd\.?$|\bLtd\.?$|\bPLC$|Association|Department|National|Society/i )
+         /\BPTY LTD$|\BLTD$|\BPLC$|ASSOCIATION|DEPARTMENT|NATIONAL|SOCIETY/ )
     {
         $name->{error} = 1;
-        $name->{properties}{non_matching} = $name->{input_string};
+        $name->{comps}{non_matching} = $name->{input_string};
+        $name->{error_desc} = 'Reserved words found in name';
     }
 
     # For the case of a single name such as 'Voltaire' we need to add a trailing space
     # to the input string. This is because the grammar tree expects a terminator (the space)
     # optionally followed by other productions or non matching text
-   if ( $name->{input_string} =~ /^[A-Z]{2,}(\-)?[A-Z]{0,}$/i )
+    $name->{input_string} .= ' ';
+    if ( $name->{input_string} =~ /^[A-Z]{2,}(\-)?[A-Z]{0,}$/ )
     {
         $name->{input_string} .= ' ';
     }
@@ -1166,103 +1121,25 @@ sub _assemble
 {
     my $name = shift;
 
-    # $::RD_TRACE  = 1;  # for debugging RecDescent output
     # Use Parse::RecDescent to do the parsing. 'full_name' is a label for the complete grammar tree
     # defined in Lingua::EN::NameParse::Grammar
     my $parsed_name = $name->{parse}->full_name($name->{input_string});
-
+    
     # Place components into a separate hash, so they can be easily returned
     # for the user to inspect and modify.
-
-    # For correct matching, the grammar of each component must include the
-    # trailing space that separates it from any following word. This should
-    # now be removed from the components, and will be restored by the
-    # case_all and salutation methods, if called.
-
-    $name->{components}{precursor} = q{};
-    if ( $parsed_name->{precursor} )
+    
+    my @all_comps = qw(precursor title_1 given_name_1 initials_1 middle_name surname_1 conjunction_1 
+    title_2 given_name_2 initials_2 surname_2 conjunction_2 suffix non_matching);
+    
+    foreach my $comp (@all_comps)
     {
-        $name->{components}{precursor} = _trim_space($parsed_name->{precursor});
-    }
-
-    $name->{components}{title_1} = q{};
-    if ( $parsed_name->{title_1} )
-    {
-        $name->{components}{title_1} = _trim_space($parsed_name->{title_1});
-    }
-
-    $name->{components}{title_2} = q{};
-    if ( $parsed_name->{title_2} )
-    {
-        $name->{components}{title_2} = _trim_space($parsed_name->{title_2});
-    }
-
-    $name->{components}{given_name_1} = q{};
-    if ( $parsed_name->{given_name_1} )
-    {
-        $name->{components}{given_name_1} = _trim_space($parsed_name->{given_name_1});
-    }
-
-    $name->{components}{given_name_2} = q{};
-    if ( $parsed_name->{given_name_2} )
-    {
-        $name->{components}{given_name_2} = _trim_space($parsed_name->{given_name_2});
-    }
-
-
-    $name->{components}{middle_name} = q{};
-    if ( $parsed_name->{middle_name} )
-    {
-        $name->{components}{middle_name} = _trim_space($parsed_name->{middle_name});
-    }
-
-    $name->{components}{initials_1} = q{};
-    if ( $parsed_name->{initials_1} )
-    {
-        $name->{components}{initials_1} = _trim_space($parsed_name->{initials_1});
-    }
-
-    $name->{components}{initials_2} = q{};
-    if ( $parsed_name->{initials_2} )
-    {
-        $name->{components}{initials_2} = _trim_space($parsed_name->{initials_2});
-    }
-
-    $name->{components}{conjunction_1} = q{};
-    if ( $parsed_name->{conjunction_1} )
-    {
-        $name->{components}{conjunction_1} = _trim_space($parsed_name->{conjunction_1});
-    }
-
-    $name->{components}{conjunction_2} = q{};
-    if ( $parsed_name->{conjunction_2} )
-    {
-        $name->{components}{conjunction_2} = _trim_space($parsed_name->{conjunction_2});
-    }
-
-    $name->{components}{surname_1} = q{};
-    if ( $parsed_name->{surname_1} )
-    {
-        $name->{components}{surname_1} = _trim_space($parsed_name->{surname_1});
-    }
-
-    $name->{components}{surname_2} = q{};
-    if ( $parsed_name->{surname_2} )
-    {
-        $name->{components}{surname_2} = _trim_space($parsed_name->{surname_2});
-    }
-
-    $name->{components}{suffix} = q{};
-    if ( $parsed_name->{suffix} )
-    {
-        $name->{components}{suffix} = _trim_space($parsed_name->{suffix});
-    }
-
-
-    $name->{properties}{non_matching} = q{};
-    if ( $parsed_name->{non_matching} )
-    {
-        $name->{properties}{non_matching}  = $parsed_name->{non_matching};
+        # set all  components to empty string, as any of them could be accessed, even if they don't exist
+         $name->{comps}{$comp} = '';
+        if (defined($parsed_name->{$comp}))
+        {
+            # Copy over existing componets.
+             $name->{comps}{$comp} = _trim_space($parsed_name->{$comp});  
+        }  
     }
 
     $name->{properties}{number} = 0;
@@ -1272,12 +1149,17 @@ sub _assemble
     return($name);
 }
 #-------------------------------------------------------------------------------
-# Remove any trailing spaces
+# For correct matching, the grammar of each component must include the trailing space that separates it
+# from any following word. This should now be removed from the components, and will be restored by the
+# case_all and salutation methods, if called.
 
 sub _trim_space
 {
     my ($string) = @_;
-    $string =~ s/ $//;
+    if ($string)
+    {
+        $string =~ s/ $//;
+    }   
     return($string);
 }
 #-------------------------------------------------------------------------------
@@ -1288,36 +1170,42 @@ sub _trim_space
 sub _validate
 {
     my $name = shift;
+     my %comps = $name->components;
 
-    if ( $name->{properties}{non_matching} )
+    if ( $comps{non_matching} )
     {
-        $name->{error} = 1;
+        $name->{warning} = 1;
+        $name->{warning_desc} .= ";non_matching text found : $comps{non_matching}";
     }
-    # illegal characters found
     elsif ( $name->{input_string} =~ /[^A-Za-z\-\'\.,&\/ ]/ )
     {
+        # illegal characters found
         $name->{error} = 1;
-    }
-    elsif ( not _valid_name($name->{components}{given_name_1}) )
+        $name->{error_desc} = 'illegal characters found';
+    }      
+    
+
+    if ( not _valid_name($comps{given_name_1}) )
     {
-        $name->{error} = 1;
+        $name->{warning} = 1;
+        $name->{warning_desc} .= ";no vowel sound in given_name_1 : $comps{given_name_1}";
     }
-    elsif ( not _valid_name($name->{components}{middle_name}) )
+    elsif ( not _valid_name($comps{middle_name}) )
     {
-        $name->{error} = 1;
+        $name->{warning} = 1;
+        $name->{warning_desc} .= ";no vowel sound in middle_name : $comps{middle_name}";
     }
 
-    elsif ( not _valid_name($name->{components}{surname_1}) )
+    elsif ( not _valid_name($comps{surname_1}) )
     {
-        $name->{error} = 1;
+        $name->{warning} = 1;
+        $name->{warning_desc} .= ";no vowel sound in surname_1 : $comps{surname_1}";
+
     }
-    elsif ( not _valid_name($name->{components}{surname_2}) )
+    elsif ( not _valid_name($comps{surname_2}) )
     {
-        $name->{error} = 1;
-    }
-    else
-    {
-        $name->{error} = 0;
+        $name->{warning} = 1;
+        $name->{warning_desc} .= ";no vowel sound in surname_2 : $comps{surname_2}";
     }
 }
 #-------------------------------------------------------------------------------
@@ -1334,7 +1222,7 @@ sub _valid_name
     }
     # Names should have a vowel sound,
     # valid exceptions are Ng, Tsz,Md, Cng,Hng,Chng etc
-    elsif ( $name and $name =~ /[aeiouyj]|^(ng|tsz|md|(c?h|[pts])ng)$/i )
+    elsif ( $name and $name =~ /[AEIOUYJ]|^(NG|TSZ|MD|(C?H|[PTS])NG)$/ )
     {
         return(1);
     }
@@ -1349,7 +1237,11 @@ sub _case_word
 {
     my ($word) = @_;
 
-    $word =~ s/(\w+)/\u\L$1/g;
+    if ($word)
+    {
+        $word =~ s/(\w+)/\u\L$1/g;
+    }    
+    
     return($word);
 }
 #-------------------------------------------------------------------------------
